@@ -1,10 +1,8 @@
 use std::cmp::max;
 
-use crate::time::{Clock, Utc};
 use borsh::{BorshDeserialize, BorshSerialize};
-
-use chrono::DateTime;
-use near_crypto::Signature;
+use chrono::{DateTime, Utc};
+use near_crypto_v01::Signature;
 use num_rational::Rational;
 use primitive_types::U256;
 
@@ -20,13 +18,14 @@ use crate::sharding::{
     ChunkHashHeight, EncodedShardChunk, ReedSolomonWrapper, ShardChunk, ShardChunkHeader,
     ShardChunkHeaderV1,
 };
-use crate::types::{Balance, BlockHeight, EpochId, Gas, NumBlocks, NumShards, StateRoot};
+#[cfg(feature = "protocol_feature_block_header_v3")]
+use crate::types::NumBlocks;
+use crate::types::{Balance, BlockHeight, EpochId, Gas, NumShards, StateRoot};
 use crate::utils::to_timestamp;
 use crate::validator_signer::{EmptyValidatorSigner, ValidatorSigner};
 use crate::version::{ProtocolVersion, SHARD_CHUNK_HEADER_UPGRADE_VERSION};
 use std::ops::Index;
 
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Clone, Debug, Eq, PartialEq, Default)]
 pub struct GenesisId {
     /// Chain Id
@@ -35,7 +34,6 @@ pub struct GenesisId {
     pub hash: CryptoHash,
 }
 
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Clone, Debug, Eq, PartialEq)]
 pub enum BlockValidityError {
     InvalidStateRoot,
@@ -46,7 +44,6 @@ pub enum BlockValidityError {
     InvalidChallengeRoot,
 }
 
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Debug, Clone, Eq, PartialEq)]
 pub struct BlockV1 {
     pub header: BlockHeader,
@@ -54,11 +51,10 @@ pub struct BlockV1 {
     pub challenges: Challenges,
 
     // Data to confirm the correctness of randomness beacon output
-    pub vrf_value: near_crypto::vrf::Value,
-    pub vrf_proof: near_crypto::vrf::Proof,
+    pub vrf_value: near_crypto_v01::vrf::Value,
+    pub vrf_proof: near_crypto_v01::vrf::Proof,
 }
 
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Debug, Clone, Eq, PartialEq)]
 pub struct BlockV2 {
     pub header: BlockHeader,
@@ -66,13 +62,12 @@ pub struct BlockV2 {
     pub challenges: Challenges,
 
     // Data to confirm the correctness of randomness beacon output
-    pub vrf_value: near_crypto::vrf::Value,
-    pub vrf_proof: near_crypto::vrf::Proof,
+    pub vrf_value: near_crypto_v01::vrf::Value,
+    pub vrf_proof: near_crypto_v01::vrf::Proof,
 }
 
 /// Versioned Block data structure.
 /// For each next version, document what are the changes between versions.
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Debug, Clone, Eq, PartialEq)]
 pub enum Block {
     BlockV1(Box<BlockV1>),
@@ -93,7 +88,7 @@ pub fn genesis_chunks(
         .map(|i| {
             let (encoded_chunk, _) = EncodedShardChunk::new(
                 CryptoHash::default(),
-                state_roots[i as usize % state_roots.len()],
+                state_roots[i as usize % state_roots.len()].clone(),
                 CryptoHash::default(),
                 genesis_height,
                 i,
@@ -104,7 +99,7 @@ pub fn genesis_chunks(
                 CryptoHash::default(),
                 vec![],
                 vec![],
-                &[],
+                &vec![],
                 CryptoHash::default(),
                 &EmptyValidatorSigner::default(),
                 genesis_protocol_version,
@@ -119,14 +114,14 @@ pub fn genesis_chunks(
 
 impl Block {
     fn block_from_protocol_version(
-        next_epoch_protocol_version: ProtocolVersion,
+        protocol_version: ProtocolVersion,
         header: BlockHeader,
         chunks: Vec<ShardChunkHeader>,
         challenges: Challenges,
-        vrf_value: near_crypto::vrf::Value,
-        vrf_proof: near_crypto::vrf::Proof,
+        vrf_value: near_crypto_v01::vrf::Value,
+        vrf_proof: near_crypto_v01::vrf::Proof,
     ) -> Block {
-        if next_epoch_protocol_version < SHARD_CHUNK_HEADER_UPGRADE_VERSION {
+        if protocol_version < SHARD_CHUNK_HEADER_UPGRADE_VERSION {
             let legacy_chunks = chunks
                 .into_iter()
                 .map(|chunk| match chunk {
@@ -134,6 +129,7 @@ impl Block {
                     ShardChunkHeader::V2(_) => panic!(
                         "Attempted to include VersionedShardChunkHeaderV2 in old protocol version"
                     ),
+                    #[cfg(feature = "protocol_feature_block_header_v3")]
                     ShardChunkHeader::V3(_) => panic!(
                         "Attempted to include VersionedShardChunkHeaderV3 in old protocol version"
                     ),
@@ -180,8 +176,8 @@ impl Block {
             initial_total_supply,
             next_bp_hash,
         );
-        let vrf_value = near_crypto::vrf::Value([0; 32]);
-        let vrf_proof = near_crypto::vrf::Proof([0; 64]);
+        let vrf_value = near_crypto_v01::vrf::Value([0; 32]);
+        let vrf_proof = near_crypto_v01::vrf::Proof([0; 64]);
 
         Self::block_from_protocol_version(
             genesis_protocol_version,
@@ -195,15 +191,16 @@ impl Block {
 
     /// Produces new block from header of previous block, current state root and set of transactions.
     pub fn produce(
-        this_epoch_protocol_version: ProtocolVersion,
-        next_epoch_protocol_version: ProtocolVersion,
+        protocol_version: ProtocolVersion,
         prev: &BlockHeader,
         height: BlockHeight,
-        block_ordinal: NumBlocks,
+        #[cfg(feature = "protocol_feature_block_header_v3")] block_ordinal: NumBlocks,
         chunks: Vec<ShardChunkHeader>,
         epoch_id: EpochId,
         next_epoch_id: EpochId,
-        epoch_sync_data_hash: Option<CryptoHash>,
+        #[cfg(feature = "protocol_feature_block_header_v3")] epoch_sync_data_hash: Option<
+            CryptoHash,
+        >,
         approvals: Vec<Option<Signature>>,
         gas_price_adjustment_rate: Rational,
         min_gas_price: Balance,
@@ -243,7 +240,8 @@ impl Block {
         );
 
         let new_total_supply = prev.total_supply() + minted_amount.unwrap_or(0) - balance_burnt;
-        let now = to_timestamp(Clock::utc());
+
+        let now = to_timestamp(Utc::now());
         let time = if now <= prev.raw_timestamp() { prev.raw_timestamp() + 1 } else { now };
 
         let (vrf_value, vrf_proof) = signer.compute_vrf_with_proof(prev.random_value().as_ref());
@@ -259,6 +257,7 @@ impl Block {
                 prev.last_final_block()
             };
 
+        #[cfg(feature = "protocol_feature_block_header_v3")]
         match prev {
             BlockHeader::BlockHeaderV1(_) => debug_assert_eq!(prev.block_ordinal(), 0),
             BlockHeader::BlockHeaderV2(_) => debug_assert_eq!(prev.block_ordinal(), 0),
@@ -268,10 +267,9 @@ impl Block {
         };
 
         let header = BlockHeader::new(
-            this_epoch_protocol_version,
-            next_epoch_protocol_version,
+            protocol_version,
             height,
-            *prev.hash(),
+            prev.hash().clone(),
             Block::compute_state_root(&chunks),
             Block::compute_chunk_receipts_root(&chunks),
             Block::compute_chunk_headers_root(&chunks).0,
@@ -282,6 +280,7 @@ impl Block {
             random_value,
             validator_proposals,
             chunk_mask,
+            #[cfg(feature = "protocol_feature_block_header_v3")]
             block_ordinal,
             epoch_id,
             next_epoch_id,
@@ -289,17 +288,19 @@ impl Block {
             new_total_supply,
             challenges_result,
             signer,
-            *last_final_block,
-            *last_ds_final_block,
+            last_final_block.clone(),
+            last_ds_final_block.clone(),
+            #[cfg(feature = "protocol_feature_block_header_v3")]
             epoch_sync_data_hash,
             approvals,
             next_bp_hash,
             block_merkle_root,
+            #[cfg(feature = "protocol_feature_block_header_v3")]
             prev.height(),
         );
 
         Self::block_from_protocol_version(
-            next_epoch_protocol_version,
+            protocol_version,
             header,
             chunks,
             challenges,
@@ -467,7 +468,7 @@ impl Block {
     }
 
     #[inline]
-    pub fn vrf_value(&self) -> &near_crypto::vrf::Value {
+    pub fn vrf_value(&self) -> &near_crypto_v01::vrf::Value {
         match self {
             Block::BlockV1(block) => &block.vrf_value,
             Block::BlockV2(block) => &block.vrf_value,
@@ -475,7 +476,7 @@ impl Block {
     }
 
     #[inline]
-    pub fn vrf_proof(&self) -> &near_crypto::vrf::Proof {
+    pub fn vrf_proof(&self) -> &near_crypto_v01::vrf::Proof {
         match self {
             Block::BlockV1(block) => &block.vrf_proof,
             Block::BlockV2(block) => &block.vrf_proof,
@@ -523,7 +524,7 @@ impl Block {
         }
 
         // Check that challenges root stored in the header matches the challenges root of the challenges
-        let challenges_root = Block::compute_challenges_root(self.challenges());
+        let challenges_root = Block::compute_challenges_root(&self.challenges());
         if self.header().challenges_root() != &challenges_root {
             return Err(InvalidChallengeRoot);
         }
@@ -584,7 +585,7 @@ impl<'a> ChunksCollection<'a> {
 
     pub fn iter<'b: 'a>(&'b self) -> VersionedChunksIter<'b> {
         match self {
-            ChunksCollection::V1(chunks) => VersionedChunksIter::new(chunks),
+            ChunksCollection::V1(chunks) => VersionedChunksIter::new(&chunks),
             ChunksCollection::V2(chunks) => VersionedChunksIter::new(chunks),
         }
     }
@@ -619,8 +620,8 @@ impl Tip {
     pub fn from_header(header: &BlockHeader) -> Tip {
         Tip {
             height: header.height(),
-            last_block_hash: *header.hash(),
-            prev_block_hash: *header.prev_hash(),
+            last_block_hash: header.hash().clone(),
+            prev_block_hash: header.prev_hash().clone(),
             epoch_id: header.epoch_id().clone(),
             next_epoch_id: header.next_epoch_id().clone(),
         }
