@@ -11,8 +11,8 @@ use crate::types::{
     ValidatorId, ValidatorKickoutReason,
 };
 use crate::version::PROTOCOL_VERSION;
-use near_primitives_core::hash::CryptoHash;
-use near_primitives_core::types::BlockHeight;
+use near_primitives_core_v01::hash::CryptoHash;
+use near_primitives_core_v01::types::BlockHeight;
 use smart_default::SmartDefault;
 use std::collections::{BTreeMap, HashMap};
 
@@ -50,7 +50,7 @@ pub struct EpochConfig {
     pub protocol_upgrade_num_epochs: EpochHeight,
     /// Shard layout of this epoch, may change from epoch to epoch
     pub shard_layout: ShardLayout,
-    /// Additional config for validator selection algorithm
+    #[cfg(feature = "protocol_feature_chunk_only_producers")]
     pub validator_selection_config: ValidatorSelectionConfig,
 }
 
@@ -61,16 +61,6 @@ pub struct ShardConfig {
     pub shard_layout: ShardLayout,
 }
 
-impl From<EpochConfig> for ShardConfig {
-    fn from(config: EpochConfig) -> Self {
-        ShardConfig {
-            num_block_producer_seats_per_shard: config.num_block_producer_seats_per_shard,
-            avg_hidden_validator_seats_per_shard: config.avg_hidden_validator_seats_per_shard,
-            shard_layout: config.shard_layout,
-        }
-    }
-}
-
 #[derive(Clone)]
 pub struct AllEpochConfig {
     genesis_epoch_config: EpochConfig,
@@ -93,11 +83,18 @@ impl AllEpochConfig {
             config.avg_hidden_validator_seats_per_shard = avg_hidden_validator_seats_per_shard;
             config.shard_layout = shard_layout;
         }
-        Self { genesis_epoch_config, simple_nightshade_epoch_config: config }
+        Self {
+            genesis_epoch_config: genesis_epoch_config.clone(),
+            simple_nightshade_epoch_config: config,
+        }
     }
 
     pub fn for_protocol_version(&self, protocol_version: ProtocolVersion) -> &EpochConfig {
-        if checked_feature!("stable", SimpleNightshade, protocol_version) {
+        if checked_feature!(
+            "protocol_feature_simple_nightshade",
+            SimpleNightshade,
+            protocol_version
+        ) {
             &self.simple_nightshade_epoch_config
         } else {
             &self.genesis_epoch_config
@@ -109,37 +106,28 @@ impl AllEpochConfig {
 /// algorithm.  See <https://github.com/near/NEPs/pull/167> for details.
 #[derive(Debug, Clone, SmartDefault, PartialEq, Eq)]
 pub struct ValidatorSelectionConfig {
-    #[cfg(feature = "protocol_feature_chunk_only_producers")]
     #[default(300)]
     pub num_chunk_only_producer_seats: NumSeats,
-    #[cfg(feature = "protocol_feature_chunk_only_producers")]
     #[default(1)]
     pub minimum_validators_per_shard: NumSeats,
     #[default(Rational::new(160, 1_000_000))]
     pub minimum_stake_ratio: Rational,
 }
 
-#[cfg(feature = "deepsize_feature")]
-impl deepsize::DeepSizeOf for ValidatorSelectionConfig {
-    fn deep_size_of_children(&self, _context: &mut deepsize::Context) -> usize {
-        0
-    }
-}
-
+#[cfg(feature = "protocol_feature_block_header_v3")]
 pub mod block_info {
     use super::SlashState;
     use crate::challenge::SlashedValidator;
     use crate::types::validator_stake::{ValidatorStake, ValidatorStakeIter};
     use crate::types::EpochId;
     use borsh::{BorshDeserialize, BorshSerialize};
-    use near_primitives_core::hash::CryptoHash;
-    use near_primitives_core::types::{AccountId, Balance, BlockHeight, ProtocolVersion};
+    use near_primitives_core_v01::hash::CryptoHash;
+    use near_primitives_core_v01::types::{AccountId, Balance, BlockHeight, ProtocolVersion};
     use std::collections::HashMap;
 
     pub use super::BlockInfoV1;
 
     /// Information per each block.
-    #[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
     #[derive(BorshSerialize, BorshDeserialize, Eq, PartialEq, Clone, Debug)]
     pub enum BlockInfo {
         V1(BlockInfoV1),
@@ -323,7 +311,6 @@ pub mod block_info {
     }
 
     // V1 -> V2: Use versioned ValidatorStake structure in proposals
-    #[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
     #[derive(Default, BorshSerialize, BorshDeserialize, Eq, PartialEq, Clone, Debug)]
     pub struct BlockInfoV2 {
         pub hash: CryptoHash,
@@ -345,8 +332,101 @@ pub mod block_info {
     }
 }
 
+#[cfg(not(feature = "protocol_feature_block_header_v3"))]
+pub mod block_info {
+    use super::SlashState;
+    use crate::types::validator_stake::ValidatorStakeIter;
+    use crate::types::EpochId;
+    use near_primitives_core_v01::hash::CryptoHash;
+    use near_primitives_core_v01::types::{AccountId, Balance, BlockHeight, ProtocolVersion};
+    use std::collections::HashMap;
+
+    pub type BlockInfo = super::BlockInfoV1;
+
+    impl BlockInfo {
+        #[inline]
+        pub fn proposals_iter(&self) -> ValidatorStakeIter {
+            ValidatorStakeIter::new(&self.proposals)
+        }
+
+        #[inline]
+        pub fn hash(&self) -> &CryptoHash {
+            &self.hash
+        }
+
+        #[inline]
+        pub fn height(&self) -> &BlockHeight {
+            &self.height
+        }
+
+        #[inline]
+        pub fn last_finalized_height(&self) -> &BlockHeight {
+            &self.last_finalized_height
+        }
+
+        #[inline]
+        pub fn last_final_block_hash(&self) -> &CryptoHash {
+            &self.last_final_block_hash
+        }
+
+        #[inline]
+        pub fn prev_hash(&self) -> &CryptoHash {
+            &self.prev_hash
+        }
+
+        #[inline]
+        pub fn epoch_first_block(&self) -> &CryptoHash {
+            &self.epoch_first_block
+        }
+
+        #[inline]
+        pub fn epoch_first_block_mut(&mut self) -> &mut CryptoHash {
+            &mut self.epoch_first_block
+        }
+
+        #[inline]
+        pub fn epoch_id(&self) -> &EpochId {
+            &self.epoch_id
+        }
+
+        #[inline]
+        pub fn epoch_id_mut(&mut self) -> &mut EpochId {
+            &mut self.epoch_id
+        }
+
+        #[inline]
+        pub fn chunk_mask(&self) -> &[bool] {
+            &self.chunk_mask
+        }
+
+        #[inline]
+        pub fn latest_protocol_version(&self) -> &ProtocolVersion {
+            &self.latest_protocol_version
+        }
+
+        #[inline]
+        pub fn slashed(&self) -> &HashMap<AccountId, SlashState> {
+            &self.slashed
+        }
+
+        #[inline]
+        pub fn slashed_mut(&mut self) -> &mut HashMap<AccountId, SlashState> {
+            &mut self.slashed
+        }
+
+        #[inline]
+        pub fn total_supply(&self) -> &Balance {
+            &self.total_supply
+        }
+
+        #[inline]
+        pub fn timestamp_nanosec(&self) -> &u64 {
+            &self.timestamp_nanosec
+        }
+    }
+}
+
 /// Information per each block.
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Default, BorshSerialize, BorshDeserialize, Eq, PartialEq, Clone, Debug)]
 pub struct BlockInfoV1 {
     pub hash: CryptoHash,
@@ -406,25 +486,27 @@ impl BlockInfoV1 {
     }
 }
 
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Default, BorshSerialize, BorshDeserialize, Clone, Debug, PartialEq, Eq)]
 pub struct ValidatorWeight(ValidatorId, u64);
 
+#[cfg(feature = "protocol_feature_block_header_v3")]
 pub mod epoch_info {
     use crate::epoch_manager::ValidatorWeight;
     use crate::types::validator_stake::{ValidatorStake, ValidatorStakeIter};
     use crate::types::{BlockChunkValidatorStats, ValidatorKickoutReason};
     use crate::version::PROTOCOL_VERSION;
     use borsh::{BorshDeserialize, BorshSerialize};
-    use near_primitives_core::hash::CryptoHash;
-    use near_primitives_core::types::{
+    use near_primitives_core_v01::hash::CryptoHash;
+    use near_primitives_core_v01::types::{
         AccountId, Balance, EpochHeight, ProtocolVersion, ValidatorId,
     };
     use smart_default::SmartDefault;
     use std::collections::{BTreeMap, HashMap};
 
+    #[cfg(feature = "protocol_feature_chunk_only_producers")]
     use crate::{checked_feature, epoch_manager::RngSeed, rand::WeightedIndex};
-    use near_primitives_core::{
+    #[cfg(feature = "protocol_feature_chunk_only_producers")]
+    use near_primitives_core_v01::{
         hash::hash,
         types::{BlockHeight, ShardId},
     };
@@ -432,11 +514,11 @@ pub mod epoch_info {
     pub use super::EpochInfoV1;
 
     /// Information per epoch.
-    #[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
     #[derive(BorshSerialize, BorshDeserialize, Clone, Debug, PartialEq, Eq)]
     pub enum EpochInfo {
         V1(EpochInfoV1),
         V2(EpochInfoV2),
+        #[cfg(feature = "protocol_feature_chunk_only_producers")]
         V3(EpochInfoV3),
     }
 
@@ -447,7 +529,6 @@ pub mod epoch_info {
     }
 
     // V1 -> V2: Use versioned ValidatorStake structure in validators and fishermen
-    #[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
     #[derive(SmartDefault, BorshSerialize, BorshDeserialize, Clone, Debug, PartialEq, Eq)]
     pub struct EpochInfoV2 {
         /// Ordinal of given epoch from genesis.
@@ -484,7 +565,7 @@ pub mod epoch_info {
 
     // V2 -> V3: Structures for randomly selecting validators at each height based on new
     // block producer and chunk producer selection algorithm.
-    #[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
+    #[cfg(feature = "protocol_feature_chunk_only_producers")]
     #[derive(SmartDefault, BorshSerialize, BorshDeserialize, Clone, Debug, PartialEq, Eq)]
     pub struct EpochInfoV3 {
         pub epoch_height: EpochHeight,
@@ -509,6 +590,42 @@ pub mod epoch_info {
     }
 
     impl EpochInfo {
+        #[cfg(not(feature = "protocol_feature_chunk_only_producers"))]
+        pub fn new(
+            epoch_height: EpochHeight,
+            validators: Vec<ValidatorStake>,
+            validator_to_index: HashMap<AccountId, ValidatorId>,
+            block_producers_settlement: Vec<ValidatorId>,
+            chunk_producers_settlement: Vec<Vec<ValidatorId>>,
+            hidden_validators_settlement: Vec<ValidatorWeight>,
+            fishermen: Vec<ValidatorStake>,
+            fishermen_to_index: HashMap<AccountId, ValidatorId>,
+            stake_change: BTreeMap<AccountId, Balance>,
+            validator_reward: HashMap<AccountId, Balance>,
+            validator_kickout: HashMap<AccountId, ValidatorKickoutReason>,
+            minted_amount: Balance,
+            seat_price: Balance,
+            protocol_version: ProtocolVersion,
+        ) -> Self {
+            Self::V2(EpochInfoV2 {
+                epoch_height,
+                validators,
+                fishermen,
+                validator_to_index,
+                block_producers_settlement,
+                chunk_producers_settlement,
+                hidden_validators_settlement,
+                stake_change,
+                validator_reward,
+                validator_kickout,
+                fishermen_to_index,
+                minted_amount,
+                seat_price,
+                protocol_version,
+            })
+        }
+
+        #[cfg(feature = "protocol_feature_chunk_only_producers")]
         pub fn new(
             epoch_height: EpochHeight,
             validators: Vec<ValidatorStake>,
@@ -526,55 +643,61 @@ pub mod epoch_info {
             protocol_version: ProtocolVersion,
             rng_seed: RngSeed,
         ) -> Self {
-            if checked_feature!("stable", AliasValidatorSelectionAlgorithm, protocol_version) {
-                let stake_weights = |ids: &[ValidatorId]| -> WeightedIndex {
-                    WeightedIndex::new(
-                        ids.iter()
-                            .copied()
-                            .map(|validator_id| validators[validator_id as usize].stake())
-                            .collect(),
-                    )
-                };
-                let block_producers_sampler = stake_weights(&block_producers_settlement);
-                let chunk_producers_sampler =
-                    chunk_producers_settlement.iter().map(|vs| stake_weights(vs)).collect();
-                Self::V3(EpochInfoV3 {
-                    epoch_height,
-                    validators,
-                    fishermen,
-                    validator_to_index,
-                    block_producers_settlement,
-                    chunk_producers_settlement,
-                    hidden_validators_settlement,
-                    stake_change,
-                    validator_reward,
-                    validator_kickout,
-                    fishermen_to_index,
-                    minted_amount,
-                    seat_price,
-                    protocol_version,
-                    rng_seed,
-                    block_producers_sampler,
-                    chunk_producers_sampler,
-                })
-            } else {
-                Self::V2(EpochInfoV2 {
-                    epoch_height,
-                    validators,
-                    fishermen,
-                    validator_to_index,
-                    block_producers_settlement,
-                    chunk_producers_settlement,
-                    hidden_validators_settlement,
-                    stake_change,
-                    validator_reward,
-                    validator_kickout,
-                    fishermen_to_index,
-                    minted_amount,
-                    seat_price,
-                    protocol_version,
-                })
-            }
+            checked_feature!(
+                "protocol_feature_chunk_only_producers",
+                ChunkOnlyProducers,
+                protocol_version,
+                {
+                    let stake_weights = |ids: &[ValidatorId]| -> WeightedIndex {
+                        WeightedIndex::new(
+                            ids.iter()
+                                .copied()
+                                .map(|validator_id| validators[validator_id as usize].stake())
+                                .collect(),
+                        )
+                    };
+                    let block_producers_sampler = stake_weights(&block_producers_settlement);
+                    let chunk_producers_sampler =
+                        chunk_producers_settlement.iter().map(|vs| stake_weights(vs)).collect();
+                    return Self::V3(EpochInfoV3 {
+                        epoch_height,
+                        validators,
+                        fishermen,
+                        validator_to_index,
+                        block_producers_settlement,
+                        chunk_producers_settlement,
+                        hidden_validators_settlement,
+                        stake_change,
+                        validator_reward,
+                        validator_kickout,
+                        fishermen_to_index,
+                        minted_amount,
+                        seat_price,
+                        protocol_version,
+                        rng_seed,
+                        block_producers_sampler,
+                        chunk_producers_sampler,
+                    });
+                },
+                {
+                    return Self::V2(EpochInfoV2 {
+                        epoch_height,
+                        validators,
+                        fishermen,
+                        validator_to_index,
+                        block_producers_settlement,
+                        chunk_producers_settlement,
+                        hidden_validators_settlement,
+                        stake_change,
+                        validator_reward,
+                        validator_kickout,
+                        fishermen_to_index,
+                        minted_amount,
+                        seat_price,
+                        protocol_version,
+                    });
+                }
+            )
         }
 
         #[inline]
@@ -582,6 +705,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => &mut v1.epoch_height,
                 Self::V2(v2) => &mut v2.epoch_height,
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => &mut v3.epoch_height,
             }
         }
@@ -591,6 +715,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => v1.epoch_height,
                 Self::V2(v2) => v2.epoch_height,
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.epoch_height,
             }
         }
@@ -600,6 +725,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => v1.seat_price,
                 Self::V2(v2) => v2.seat_price,
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.seat_price,
             }
         }
@@ -609,6 +735,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => v1.minted_amount,
                 Self::V2(v2) => v2.minted_amount,
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.minted_amount,
             }
         }
@@ -618,6 +745,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => &v1.block_producers_settlement,
                 Self::V2(v2) => &v2.block_producers_settlement,
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => &v3.block_producers_settlement,
             }
         }
@@ -627,6 +755,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => &v1.chunk_producers_settlement,
                 Self::V2(v2) => &v2.chunk_producers_settlement,
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => &v3.chunk_producers_settlement,
             }
         }
@@ -636,6 +765,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => &v1.validator_kickout,
                 Self::V2(v2) => &v2.validator_kickout,
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => &v3.validator_kickout,
             }
         }
@@ -645,6 +775,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => v1.protocol_version,
                 Self::V2(v2) => v2.protocol_version,
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.protocol_version,
             }
         }
@@ -654,6 +785,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => &v1.stake_change,
                 Self::V2(v2) => &v2.stake_change,
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => &v3.stake_change,
             }
         }
@@ -663,6 +795,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => &v1.validator_reward,
                 Self::V2(v2) => &v2.validator_reward,
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => &v3.validator_reward,
             }
         }
@@ -672,6 +805,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => ValidatorStakeIter::v1(&v1.validators),
                 Self::V2(v2) => ValidatorStakeIter::new(&v2.validators),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => ValidatorStakeIter::new(&v3.validators),
             }
         }
@@ -681,6 +815,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => ValidatorStakeIter::v1(&v1.fishermen),
                 Self::V2(v2) => ValidatorStakeIter::new(&v2.fishermen),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => ValidatorStakeIter::new(&v3.fishermen),
             }
         }
@@ -690,6 +825,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => v1.validators[validator_id as usize].stake,
                 Self::V2(v2) => v2.validators[validator_id as usize].stake(),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.validators[validator_id as usize].stake(),
             }
         }
@@ -699,6 +835,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => &v1.validators[validator_id as usize].account_id,
                 Self::V2(v2) => v2.validators[validator_id as usize].account_id(),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.validators[validator_id as usize].account_id(),
             }
         }
@@ -708,6 +845,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => v1.validator_to_index.contains_key(account_id),
                 Self::V2(v2) => v2.validator_to_index.contains_key(account_id),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.validator_to_index.contains_key(account_id),
             }
         }
@@ -716,6 +854,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => v1.validator_to_index.get(account_id),
                 Self::V2(v2) => v2.validator_to_index.get(account_id),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.validator_to_index.get(account_id),
             }
         }
@@ -729,6 +868,7 @@ pub mod epoch_info {
                     .validator_to_index
                     .get(account_id)
                     .map(|validator_id| v2.validators[*validator_id as usize].clone()),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3
                     .validator_to_index
                     .get(account_id)
@@ -741,6 +881,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => ValidatorStake::V1(v1.validators[validator_id as usize].clone()),
                 Self::V2(v2) => v2.validators[validator_id as usize].clone(),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.validators[validator_id as usize].clone(),
             }
         }
@@ -750,6 +891,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => v1.fishermen_to_index.contains_key(account_id),
                 Self::V2(v2) => v2.fishermen_to_index.contains_key(account_id),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.fishermen_to_index.contains_key(account_id),
             }
         }
@@ -763,6 +905,7 @@ pub mod epoch_info {
                     .fishermen_to_index
                     .get(account_id)
                     .map(|validator_id| v2.fishermen[*validator_id as usize].clone()),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3
                     .fishermen_to_index
                     .get(account_id)
@@ -775,6 +918,7 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => ValidatorStake::V1(v1.fishermen[fisherman_id as usize].clone()),
                 Self::V2(v2) => v2.fishermen[fisherman_id as usize].clone(),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.fishermen[fisherman_id as usize].clone(),
             }
         }
@@ -784,10 +928,12 @@ pub mod epoch_info {
             match self {
                 Self::V1(v1) => v1.validators.len(),
                 Self::V2(v2) => v2.validators.len(),
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V3(v3) => v3.validators.len(),
             }
         }
 
+        #[cfg(feature = "protocol_feature_chunk_only_producers")]
         pub fn sample_block_producer(&self, height: BlockHeight) -> ValidatorId {
             match &self {
                 Self::V1(v1) => {
@@ -799,12 +945,18 @@ pub mod epoch_info {
                     bp_settlement[(height % (bp_settlement.len() as u64)) as usize]
                 }
                 Self::V3(v3) => {
-                    let seed = Self::block_produce_seed(height, &v3.rng_seed);
+                    let seed = {
+                        let mut buffer = [0u8; 40]; // 32 bytes from epoch_seed, 8 bytes from height
+                        buffer[0..32].copy_from_slice(&v3.rng_seed);
+                        buffer[32..40].copy_from_slice(&height.to_le_bytes());
+                        hash(&buffer).0
+                    };
                     v3.block_producers_settlement[v3.block_producers_sampler.sample(seed)]
                 }
             }
         }
 
+        #[cfg(feature = "protocol_feature_chunk_only_producers")]
         pub fn sample_chunk_producer(&self, height: BlockHeight, shard_id: ShardId) -> ValidatorId {
             match &self {
                 Self::V1(v1) => {
@@ -818,19 +970,7 @@ pub mod epoch_info {
                     shard_cps[(height as u64 % (shard_cps.len() as u64)) as usize]
                 }
                 Self::V3(v3) => {
-                    let protocol_version = self.protocol_version();
-                    let seed = if checked_feature!(
-                        "stable",
-                        SynchronizeBlockChunkProduction,
-                        protocol_version
-                    ) && !checked_feature!(
-                        "protocol_feature_chunk_only_producers",
-                        ChunkOnlyProducers,
-                        protocol_version
-                    ) {
-                        // This is same seed that used for determining block producer
-                        Self::block_produce_seed(height, &v3.rng_seed)
-                    } else {
+                    let seed = {
                         // 32 bytes from epoch_seed, 8 bytes from height, 8 bytes from shard_id
                         let mut buffer = [0u8; 48];
                         buffer[0..32].copy_from_slice(&v3.rng_seed);
@@ -844,13 +984,182 @@ pub mod epoch_info {
                 }
             }
         }
+    }
 
-        /// 32 bytes from epoch_seed, 8 bytes from height
-        fn block_produce_seed(height: BlockHeight, seed: &RngSeed) -> [u8; 32] {
-            let mut buffer = [0u8; 40];
-            buffer[0..32].copy_from_slice(seed);
-            buffer[32..40].copy_from_slice(&height.to_le_bytes());
-            hash(&buffer).0
+    #[derive(BorshSerialize, BorshDeserialize)]
+    pub struct EpochSummary {
+        pub prev_epoch_last_block_hash: CryptoHash,
+        /// Proposals from the epoch, only the latest one per account
+        pub all_proposals: Vec<ValidatorStake>,
+        /// Kickout set, includes slashed
+        pub validator_kickout: HashMap<AccountId, ValidatorKickoutReason>,
+        /// Only for validators who met the threshold and didn't get slashed
+        pub validator_block_chunk_stats: HashMap<AccountId, BlockChunkValidatorStats>,
+        /// Protocol version for next epoch.
+        pub next_version: ProtocolVersion,
+    }
+}
+
+#[cfg(not(feature = "protocol_feature_block_header_v3"))]
+pub mod epoch_info {
+    use crate::epoch_manager::ValidatorWeight;
+    use crate::types::validator_stake::{ValidatorStake, ValidatorStakeIter};
+    use crate::types::{BlockChunkValidatorStats, ValidatorKickoutReason};
+    use borsh::{BorshDeserialize, BorshSerialize};
+    use near_primitives_core_v01::hash::CryptoHash;
+    use near_primitives_core_v01::types::{
+        AccountId, Balance, EpochHeight, ProtocolVersion, ValidatorId,
+    };
+    use std::collections::{BTreeMap, HashMap};
+
+    pub use super::EpochInfoV1;
+    pub type EpochInfo = super::EpochInfoV1;
+
+    impl EpochInfo {
+        pub fn new(
+            epoch_height: EpochHeight,
+            validators: Vec<ValidatorStake>,
+            validator_to_index: HashMap<AccountId, ValidatorId>,
+            block_producers_settlement: Vec<ValidatorId>,
+            chunk_producers_settlement: Vec<Vec<ValidatorId>>,
+            hidden_validators_settlement: Vec<ValidatorWeight>,
+            fishermen: Vec<ValidatorStake>,
+            fishermen_to_index: HashMap<AccountId, ValidatorId>,
+            stake_change: BTreeMap<AccountId, Balance>,
+            validator_reward: HashMap<AccountId, Balance>,
+            validator_kickout: HashMap<AccountId, ValidatorKickoutReason>,
+            minted_amount: Balance,
+            seat_price: Balance,
+            protocol_version: ProtocolVersion,
+        ) -> Self {
+            Self {
+                epoch_height,
+                validators,
+                fishermen,
+                validator_to_index,
+                block_producers_settlement,
+                chunk_producers_settlement,
+                hidden_validators_settlement,
+                stake_change,
+                validator_reward,
+                validator_kickout,
+                fishermen_to_index,
+                minted_amount,
+                seat_price,
+                protocol_version,
+            }
+        }
+
+        #[inline]
+        pub fn epoch_height_mut(&mut self) -> &mut EpochHeight {
+            &mut self.epoch_height
+        }
+
+        #[inline]
+        pub fn epoch_height(&self) -> EpochHeight {
+            self.epoch_height
+        }
+
+        #[inline]
+        pub fn seat_price(&self) -> Balance {
+            self.seat_price
+        }
+
+        #[inline]
+        pub fn minted_amount(&self) -> Balance {
+            self.minted_amount
+        }
+
+        #[inline]
+        pub fn block_producers_settlement(&self) -> &[ValidatorId] {
+            &self.block_producers_settlement
+        }
+
+        #[inline]
+        pub fn chunk_producers_settlement(&self) -> &[Vec<ValidatorId>] {
+            &self.chunk_producers_settlement
+        }
+
+        #[inline]
+        pub fn validator_kickout(&self) -> &HashMap<AccountId, ValidatorKickoutReason> {
+            &self.validator_kickout
+        }
+
+        #[inline]
+        pub fn protocol_version(&self) -> ProtocolVersion {
+            self.protocol_version
+        }
+
+        #[inline]
+        pub fn stake_change(&self) -> &BTreeMap<AccountId, Balance> {
+            &self.stake_change
+        }
+
+        #[inline]
+        pub fn validator_reward(&self) -> &HashMap<AccountId, Balance> {
+            &self.validator_reward
+        }
+
+        #[inline]
+        pub fn validators_iter(&self) -> ValidatorStakeIter {
+            ValidatorStakeIter::new(&self.validators)
+        }
+
+        #[inline]
+        pub fn fishermen_iter(&self) -> ValidatorStakeIter {
+            ValidatorStakeIter::new(&self.fishermen)
+        }
+
+        #[inline]
+        pub fn validator_stake(&self, validator_id: u64) -> Balance {
+            self.validators[validator_id as usize].stake
+        }
+
+        #[inline]
+        pub fn validator_account_id(&self, validator_id: u64) -> &AccountId {
+            &self.validators[validator_id as usize].account_id
+        }
+
+        #[inline]
+        pub fn account_is_validator(&self, account_id: &AccountId) -> bool {
+            self.validator_to_index.contains_key(account_id)
+        }
+
+        #[inline]
+        pub fn get_validator_id(&self, account_id: &AccountId) -> Option<&ValidatorId> {
+            self.validator_to_index.get(account_id)
+        }
+
+        pub fn get_validator_by_account(&self, account_id: &AccountId) -> Option<ValidatorStake> {
+            self.validator_to_index
+                .get(account_id)
+                .map(|validator_id| self.validators[*validator_id as usize].clone())
+        }
+
+        #[inline]
+        pub fn get_validator(&self, validator_id: u64) -> ValidatorStake {
+            self.validators[validator_id as usize].clone()
+        }
+
+        #[inline]
+        pub fn account_is_fisherman(&self, account_id: &AccountId) -> bool {
+            self.fishermen_to_index.contains_key(account_id)
+        }
+
+        pub fn get_fisherman_by_account(&self, account_id: &AccountId) -> Option<ValidatorStake> {
+            self.fishermen_to_index
+                .get(account_id)
+                .map(|validator_id| self.fishermen[*validator_id as usize].clone())
+        }
+
+        #[inline]
+        pub fn get_fisherman(&self, fisherman_id: u64) -> ValidatorStake {
+            self.fishermen[fisherman_id as usize].clone()
+        }
+
+        #[inline]
+        pub fn validators_len(&self) -> usize {
+            self.validators.len()
         }
     }
 
@@ -869,7 +1178,6 @@ pub mod epoch_info {
 }
 
 /// Information per epoch.
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(SmartDefault, BorshSerialize, BorshDeserialize, Clone, Debug, PartialEq, Eq)]
 pub struct EpochInfoV1 {
     /// Ordinal of given epoch from genesis.
@@ -905,7 +1213,6 @@ pub struct EpochInfoV1 {
 }
 
 /// State that a slashed validator can be in.
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq)]
 pub enum SlashState {
     /// Double Sign, will be partially slashed.
