@@ -1,19 +1,20 @@
-pub use near_primitives_core::borsh;
-pub use near_primitives_core::num_rational;
+pub use near_primitives_core_v01::borsh;
+pub use near_primitives_core_v01::num_rational;
 
-pub use near_primitives_core::account;
+pub use near_primitives_core_v01::account;
 pub mod block;
 pub mod block_header;
 pub mod challenge;
-pub use near_primitives_core::config;
-pub use near_primitives_core::contract;
+pub use near_primitives_core_v01::config;
+pub use near_primitives_core_v01::contract;
 pub mod epoch_manager;
 pub mod errors;
-pub use near_primitives_core::hash;
-pub use near_primitives_core::logging;
+pub use near_primitives_core_v01::hash;
+pub use near_primitives_core_v01::logging;
 pub mod merkle;
 pub mod network;
-pub use near_primitives_core::profile;
+pub use near_primitives_core_v01::profile;
+#[cfg(feature = "protocol_feature_chunk_only_producers")]
 pub mod rand;
 pub mod receipt;
 pub mod runtime;
@@ -24,7 +25,6 @@ pub mod state_record;
 pub mod syncing;
 pub mod telemetry;
 pub mod test_utils;
-pub mod time;
 pub mod transaction;
 pub mod trie_key;
 pub mod types;
