@@ -1,5 +1,5 @@
-pub use near_primitives_core::runtime::fees;
-pub use near_primitives_core::runtime::*;
+pub use near_primitives_core_v01::runtime::fees;
+pub use near_primitives_core_v01::runtime::*;
 
 use crate::account::Account;
 use crate::runtime::config::RuntimeConfig;
