@@ -2,12 +2,15 @@ use std::collections::HashMap;
 
 use num_rational::Rational;
 
-use near_crypto::{EmptySigner, PublicKey, Signature, Signer};
-use near_primitives_core::types::ProtocolVersion;
+use near_crypto_v01::{EmptySigner, PublicKey, Signature, Signer};
 
 use crate::account::{AccessKey, AccessKeyPermission, Account};
 use crate::block::Block;
-use crate::block_header::{BlockHeader, BlockHeaderV3};
+use crate::block_header::BlockHeader;
+#[cfg(not(feature = "protocol_feature_block_header_v3"))]
+use crate::block_header::BlockHeaderV2;
+#[cfg(feature = "protocol_feature_block_header_v3")]
+use crate::block_header::BlockHeaderV3;
 use crate::errors::{EpochError, TxExecutionError};
 use crate::hash::CryptoHash;
 use crate::merkle::PartialMerkleTree;
@@ -246,8 +249,8 @@ impl SignedTransaction {
     pub fn empty(block_hash: CryptoHash) -> Self {
         Self::from_actions(
             0,
-            "test".parse().unwrap(),
-            "test".parse().unwrap(),
+            AccountId::test_account(),
+            AccountId::test_account(),
             &EmptySigner {},
             vec![],
             block_hash,
@@ -256,6 +259,7 @@ impl SignedTransaction {
 }
 
 impl BlockHeader {
+    #[cfg(feature = "protocol_feature_block_header_v3")]
     pub fn get_mut(&mut self) -> &mut BlockHeaderV3 {
         match self {
             BlockHeader::BlockHeaderV1(_) | BlockHeader::BlockHeaderV2(_) => {
@@ -265,17 +269,11 @@ impl BlockHeader {
         }
     }
 
-    pub fn set_lastest_protocol_version(&mut self, latest_protocol_version: ProtocolVersion) {
+    #[cfg(not(feature = "protocol_feature_block_header_v3"))]
+    pub fn get_mut(&mut self) -> &mut BlockHeaderV2 {
         match self {
-            BlockHeader::BlockHeaderV1(header) => {
-                header.inner_rest.latest_protocol_version = latest_protocol_version;
-            }
-            BlockHeader::BlockHeaderV2(header) => {
-                header.inner_rest.latest_protocol_version = latest_protocol_version;
-            }
-            BlockHeader::BlockHeaderV3(header) => {
-                header.inner_rest.latest_protocol_version = latest_protocol_version;
-            }
+            BlockHeader::BlockHeaderV1(_) => panic!("old header should not appear in tests"),
+            BlockHeader::BlockHeaderV2(header) => header,
         }
     }
 
@@ -285,20 +283,9 @@ impl BlockHeader {
             &self.inner_lite_bytes(),
             &self.inner_rest_bytes(),
         );
-        match self {
-            BlockHeader::BlockHeaderV1(header) => {
-                header.hash = hash;
-                header.signature = signature;
-            }
-            BlockHeader::BlockHeaderV2(header) => {
-                header.hash = hash;
-                header.signature = signature;
-            }
-            BlockHeader::BlockHeaderV3(header) => {
-                header.hash = hash;
-                header.signature = signature;
-            }
-        }
+        let mut header = self.get_mut();
+        header.hash = hash;
+        header.signature = signature;
     }
 }
 
@@ -320,6 +307,7 @@ impl Block {
                         ShardChunkHeader::V2(_) => {
                             panic!("Attempted to set V1 block chunks with V2")
                         }
+                        #[cfg(feature = "protocol_feature_block_header_v3")]
                         ShardChunkHeader::V3(_) => {
                             panic!("Attempted to set V1 block chunks with V3")
                         }
@@ -419,14 +407,15 @@ impl Block {
         block_merkle_root: CryptoHash,
     ) -> Self {
         Block::produce(
-            PROTOCOL_VERSION,
             PROTOCOL_VERSION,
             prev.header(),
             height,
-            prev.header().block_ordinal() + 1,
+            #[cfg(feature = "protocol_feature_block_header_v3")]
+            (prev.header().block_ordinal() + 1),
             prev.chunks().iter().cloned().collect(),
             epoch_id,
             next_epoch_id,
+            #[cfg(feature = "protocol_feature_block_header_v3")]
             None,
             approvals,
             Rational::from_integer(0),
