@@ -1,7 +1,7 @@
 use crate::hash::CryptoHash;
 use crate::types::AccountId;
 use borsh::{BorshDeserialize, BorshSerialize};
-use near_crypto::PublicKey;
+use near_crypto_v01::PublicKey;
 use std::mem::size_of;
 
 pub(crate) const ACCOUNT_DATA_SEPARATOR: &[u8; 1] = b",";
@@ -192,6 +192,7 @@ impl TrieKey {
 // TODO: Remove once we switch to non-raw keys everywhere.
 pub mod trie_key_parsers {
     use super::*;
+    use std::convert::TryFrom;
 
     pub fn parse_public_key_from_access_key_key(
         raw_key: &[u8],
@@ -327,7 +328,7 @@ pub mod trie_key_parsers {
     pub fn parse_account_id_from_raw_key(
         raw_key: &[u8],
     ) -> Result<Option<AccountId>, std::io::Error> {
-        for (col, col_name) in col::NON_DELAYED_RECEIPT_COLUMNS {
+        for (col, col_name) in col::NON_DELAYED_RECEIPT_COLUMNS.into_iter() {
             if parse_account_id_prefix(col, raw_key).is_err() {
                 continue;
             }
@@ -416,7 +417,7 @@ pub mod trie_key_parsers {
 
 #[cfg(test)]
 mod tests {
-    use near_crypto::KeyType;
+    use near_crypto_v01::KeyType;
 
     use super::*;
 
