@@ -2,7 +2,7 @@ use borsh::{BorshDeserialize, BorshSerialize};
 use derive_more::{AsRef as DeriveAsRef, From as DeriveFrom};
 use serde::{Deserialize, Serialize};
 
-use near_crypto::PublicKey;
+use near_crypto_v01::PublicKey;
 
 use crate::account::{AccessKey, Account};
 use crate::challenge::ChallengesResult;
@@ -13,13 +13,12 @@ use crate::trie_key::TrieKey;
 
 use crate::receipt::Receipt;
 /// Reexport primitive types
-pub use near_primitives_core::types::*;
+pub use near_primitives_core_v01::types::*;
 
 /// Hash used by to store state root.
 pub type StateRoot = CryptoHash;
 
 /// Different types of finality.
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Serialize, Deserialize, BorshSerialize, BorshDeserialize, Clone, Debug, PartialEq, Eq)]
 pub enum Finality {
     #[serde(rename = "optimistic")]
@@ -55,7 +54,6 @@ pub struct AccountInfo {
 ///
 /// NOTE: Currently, this type is only used in the view_client and RPC to be able to transparently
 /// pretty-serialize the bytes arrays as base64-encoded strings (see `serialize.rs`).
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(
     Debug, Clone, PartialEq, Eq, DeriveAsRef, DeriveFrom, BorshSerialize, BorshDeserialize,
 )]
@@ -77,7 +75,6 @@ pub struct StoreValue(Vec<u8>);
 /// NOTE: The main reason for this to exist (except the type-safety) is that the value is
 /// transparently serialized and deserialized as a base64-encoded string when serde is used
 /// (serde_json).
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(
     Debug, Clone, PartialEq, Eq, DeriveAsRef, DeriveFrom, BorshSerialize, BorshDeserialize,
 )]
@@ -291,14 +288,15 @@ impl StateChanges {
                     state_changes.extend(changes.into_iter().map(
                         |RawStateChange { cause, data }| StateChangeWithCause {
                             cause,
-                            value: match data {
-                                Some(change_data) => StateChangeValue::ContractCodeUpdate {
+                            value: if let Some(change_data) = data {
+                                StateChangeValue::ContractCodeUpdate {
                                     account_id: account_id.clone(),
-                                    code: change_data,
-                                },
-                                None => StateChangeValue::ContractCodeDeletion {
+                                    code: change_data.into(),
+                                }
+                            } else {
+                                StateChangeValue::ContractCodeDeletion {
                                     account_id: account_id.clone(),
-                                },
+                                }
                             },
                         },
                     ));
@@ -402,7 +400,6 @@ impl StateChanges {
     }
 }
 
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(PartialEq, Eq, Clone, Debug, BorshSerialize, BorshDeserialize, Serialize)]
 pub struct StateRootNode {
     /// in Nightshade, data is the serialized TrieNodeWithSize
@@ -420,7 +417,6 @@ impl StateRootNode {
 /// Epoch identifier -- wrapped hash, to make it easier to distinguish.
 /// EpochId of epoch T is the hash of last block in T-2
 /// EpochId of first two epochs is 0
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(
     Debug,
     Clone,
@@ -441,7 +437,6 @@ pub struct EpochId(pub CryptoHash);
 /// Stores validator and its stake for two consecutive epochs.
 /// It is necessary because the blocks on the epoch boundary need to contain approvals from both
 /// epochs.
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq)]
 pub struct ApprovalStake {
     /// Account that stakes money.
@@ -453,17 +448,17 @@ pub struct ApprovalStake {
     pub stake_next_epoch: Balance,
 }
 
+#[cfg(feature = "protocol_feature_block_header_v3")]
 pub mod validator_stake {
     use crate::types::ApprovalStake;
     use borsh::{BorshDeserialize, BorshSerialize};
-    use near_crypto::PublicKey;
-    use near_primitives_core::types::{AccountId, Balance};
+    use near_crypto_v01::PublicKey;
+    use near_primitives_core_v01::types::{AccountId, Balance};
     use serde::Serialize;
 
     pub use super::ValidatorStakeV1;
 
     /// Stores validator and its stake.
-    #[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
     #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq)]
     #[serde(tag = "validator_stake_struct_version")]
     pub enum ValidatorStake {
@@ -472,7 +467,6 @@ pub mod validator_stake {
         V2(ValidatorStakeV2),
     }
 
-    #[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
     #[cfg(feature = "protocol_feature_chunk_only_producers")]
     #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq)]
     pub struct ValidatorStakeV2 {
@@ -568,9 +562,9 @@ pub mod validator_stake {
                 Self::V1(v1) => v1,
                 #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V2(v2) => {
-                    // This function is called on V2 variant if
-                    // protocol_feature_chunk_only_producers is enabled, but current protocol
-                    // version is lower than required for block headers v3.
+                    // This function should never be called on a V2 variant, it
+                    // is only for backwards compatibility purposes.
+                    debug_assert!(false);
                     ValidatorStakeV1 {
                         account_id: v2.account_id,
                         public_key: v2.public_key,
@@ -580,11 +574,11 @@ pub mod validator_stake {
             }
         }
 
+        #[cfg(feature = "protocol_feature_chunk_only_producers")]
         #[inline]
         pub fn is_chunk_only(&self) -> bool {
             match self {
                 Self::V1(_) => false,
-                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V2(v2) => v2.is_chunk_only,
             }
         }
@@ -672,8 +666,108 @@ pub mod validator_stake {
     }
 }
 
+#[cfg(not(feature = "protocol_feature_block_header_v3"))]
+pub mod validator_stake {
+    use crate::types::ApprovalStake;
+    use near_crypto_v01::PublicKey;
+    use near_primitives_core_v01::types::{AccountId, Balance};
+
+    pub use super::ValidatorStakeV1;
+    pub type ValidatorStake = ValidatorStakeV1;
+
+    pub struct ValidatorStakeIter<'a> {
+        inner: std::slice::Iter<'a, ValidatorStake>,
+        len: usize,
+    }
+
+    impl<'a> ValidatorStakeIter<'a> {
+        pub fn empty() -> Self {
+            Self { inner: (&[]).iter(), len: 0 }
+        }
+
+        pub fn v1(collection: &'a [ValidatorStakeV1]) -> Self {
+            Self::new(collection)
+        }
+
+        pub fn new(collection: &'a [ValidatorStake]) -> Self {
+            Self { inner: collection.iter(), len: collection.len() }
+        }
+
+        pub fn len(&self) -> usize {
+            self.len
+        }
+    }
+
+    impl<'a> Iterator for ValidatorStakeIter<'a> {
+        type Item = ValidatorStake;
+
+        fn next(&mut self) -> Option<Self::Item> {
+            self.inner.next().cloned()
+        }
+    }
+
+    impl ValidatorStake {
+        pub fn new(account_id: AccountId, public_key: PublicKey, stake: Balance) -> Self {
+            ValidatorStake { account_id, public_key, stake }
+        }
+
+        #[inline]
+        pub fn account_and_stake(self) -> (AccountId, Balance) {
+            (self.account_id, self.stake)
+        }
+
+        #[inline]
+        pub fn destructure(self) -> (AccountId, PublicKey, Balance) {
+            (self.account_id, self.public_key, self.stake)
+        }
+
+        #[inline]
+        pub fn into_v1(self) -> ValidatorStakeV1 {
+            self
+        }
+
+        #[inline]
+        pub fn take_account_id(self) -> AccountId {
+            self.account_id
+        }
+
+        #[inline]
+        pub fn account_id(&self) -> &AccountId {
+            &self.account_id
+        }
+
+        #[inline]
+        pub fn take_public_key(self) -> PublicKey {
+            self.public_key
+        }
+
+        #[inline]
+        pub fn public_key(&self) -> &PublicKey {
+            &self.public_key
+        }
+
+        #[inline]
+        pub fn stake(&self) -> Balance {
+            self.stake
+        }
+
+        #[inline]
+        pub fn stake_mut(&mut self) -> &mut Balance {
+            &mut self.stake
+        }
+
+        pub fn get_approval_stake(&self, is_next_epoch: bool) -> ApprovalStake {
+            ApprovalStake {
+                account_id: self.account_id.clone(),
+                public_key: self.public_key.clone(),
+                stake_this_epoch: if is_next_epoch { 0 } else { self.stake },
+                stake_next_epoch: if is_next_epoch { self.stake } else { 0 },
+            }
+        }
+    }
+}
+
 /// Stores validator and its stake.
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq)]
 pub struct ValidatorStakeV1 {
     /// Account that stakes money.
@@ -685,18 +779,18 @@ pub struct ValidatorStakeV1 {
 }
 
 /// Information after block was processed.
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Debug, PartialEq, BorshSerialize, BorshDeserialize, Clone, Eq)]
 pub struct BlockExtra {
     pub challenges_result: ChallengesResult,
 }
 
+#[cfg(feature = "protocol_feature_block_header_v3")]
 pub mod chunk_extra {
     use crate::types::validator_stake::{ValidatorStake, ValidatorStakeIter};
     use crate::types::StateRoot;
     use borsh::{BorshDeserialize, BorshSerialize};
-    use near_primitives_core::hash::CryptoHash;
-    use near_primitives_core::types::{Balance, Gas};
+    use near_primitives_core_v01::hash::CryptoHash;
+    use near_primitives_core_v01::types::{Balance, Gas};
 
     pub use super::ChunkExtraV1;
 
@@ -737,7 +831,7 @@ pub mod chunk_extra {
             balance_burnt: Balance,
         ) -> Self {
             Self::V2(ChunkExtraV2 {
-                state_root: *state_root,
+                state_root: state_root.clone(),
                 outcome_root,
                 validator_proposals,
                 gas_used,
@@ -804,6 +898,76 @@ pub mod chunk_extra {
     }
 }
 
+#[cfg(not(feature = "protocol_feature_block_header_v3"))]
+pub mod chunk_extra {
+    use crate::types::validator_stake::{ValidatorStake, ValidatorStakeIter};
+    use crate::types::StateRoot;
+    use near_primitives_core_v01::hash::CryptoHash;
+    use near_primitives_core_v01::types::{Balance, Gas};
+
+    pub use super::ChunkExtraV1;
+    pub type ChunkExtra = ChunkExtraV1;
+
+    impl ChunkExtra {
+        pub fn new_with_only_state_root(state_root: &StateRoot) -> Self {
+            Self::new(state_root, CryptoHash::default(), vec![], 0, 0, 0)
+        }
+
+        pub fn new(
+            state_root: &StateRoot,
+            outcome_root: CryptoHash,
+            validator_proposals: Vec<ValidatorStake>,
+            gas_used: Gas,
+            gas_limit: Gas,
+            balance_burnt: Balance,
+        ) -> Self {
+            Self {
+                state_root: state_root.clone(),
+                outcome_root,
+                validator_proposals,
+                gas_used,
+                gas_limit,
+                balance_burnt,
+            }
+        }
+
+        #[inline]
+        pub fn outcome_root(&self) -> &StateRoot {
+            &self.outcome_root
+        }
+
+        #[inline]
+        pub fn state_root(&self) -> &StateRoot {
+            &self.state_root
+        }
+
+        #[inline]
+        pub fn state_root_mut(&mut self) -> &mut StateRoot {
+            &mut self.state_root
+        }
+
+        #[inline]
+        pub fn validator_proposals(&self) -> ValidatorStakeIter {
+            ValidatorStakeIter::new(&self.validator_proposals)
+        }
+
+        #[inline]
+        pub fn gas_limit(&self) -> Gas {
+            self.gas_limit
+        }
+
+        #[inline]
+        pub fn gas_used(&self) -> Gas {
+            self.gas_used
+        }
+
+        #[inline]
+        pub fn balance_burnt(&self) -> Balance {
+            self.balance_burnt
+        }
+    }
+}
+
 /// Information after chunk was processed, used to produce or check next chunk.
 #[derive(Debug, PartialEq, BorshSerialize, BorshDeserialize, Clone, Eq)]
 pub struct ChunkExtraV1 {
@@ -821,7 +985,6 @@ pub struct ChunkExtraV1 {
     pub balance_burnt: Balance,
 }
 
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Debug, Clone, PartialEq, Eq, BorshSerialize, BorshDeserialize, Serialize, Deserialize)]
 #[serde(untagged)]
 pub enum BlockId {
@@ -831,7 +994,6 @@ pub enum BlockId {
 
 pub type MaybeBlockId = Option<BlockId>;
 
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Debug, Clone, PartialEq, Eq, BorshSerialize, BorshDeserialize, Serialize, Deserialize)]
 #[serde(rename_all = "snake_case")]
 pub enum SyncCheckpoint {
@@ -839,7 +1001,6 @@ pub enum SyncCheckpoint {
     EarliestAvailable,
 }
 
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Debug, Clone, PartialEq, Eq, BorshSerialize, BorshDeserialize, Serialize, Deserialize)]
 #[serde(rename_all = "snake_case")]
 pub enum BlockReference {
@@ -906,7 +1067,6 @@ impl Serialize for EpochReference {
 }
 
 /// Reasons for removing a validator from the validator set.
-#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Clone, Debug, PartialEq, Eq)]
 pub enum ValidatorKickoutReason {
     /// Slashed validators are kicked out.
