@@ -1,5 +1,6 @@
 use std::cmp::Reverse;
 use std::collections::binary_heap::{BinaryHeap, IntoIter};
+use std::iter::FromIterator;
 
 /// Wrapper around `BinaryHeap` to be default min heap instead of max heap.
 #[derive(Debug, Clone)]
@@ -19,10 +20,6 @@ impl<T: Ord> MinHeap<T> {
     pub fn peek(&self) -> Option<&T> {
         self.inner.peek().map(|Reverse(t)| t)
     }
-
-    pub fn len(&self) -> usize {
-        self.inner.len()
-    }
 }
 
 impl<T> Default for MinHeap<T>
@@ -64,12 +61,10 @@ mod tests {
         heap.push(4);
         heap.push(1);
 
-        assert_eq!(heap.len(), 4);
         assert_eq!(heap.pop(), Some(1));
         assert_eq!(heap.pop(), Some(4));
         assert_eq!(heap.pop(), Some(7));
         assert_eq!(heap.pop(), Some(11));
-        assert_eq!(heap.len(), 0);
         assert_eq!(heap.pop(), None);
     }
 
@@ -83,15 +78,12 @@ mod tests {
         heap.push(2);
         heap.push(1);
 
-        assert_eq!(heap.len(), 4);
         assert_eq!(heap.pop(), Some(1));
         assert_eq!(heap.pop(), Some(2));
-        assert_eq!(heap.len(), 2);
 
         heap.push(4);
         heap.push(32);
 
-        assert_eq!(heap.len(), 4);
         assert_eq!(heap.pop(), Some(4));
         assert_eq!(heap.pop(), Some(8));
         assert_eq!(heap.pop(), Some(16));
@@ -104,7 +96,6 @@ mod tests {
         // Elements collected into a heap should be popped back in increasing order.
         let mut heap: MinHeap<usize> = [9, 3, 100, 10, 5].iter().copied().collect();
 
-        assert_eq!(heap.len(), 5);
         assert_eq!(heap.pop(), Some(3));
         assert_eq!(heap.pop(), Some(5));
         assert_eq!(heap.pop(), Some(9));
