@@ -3,7 +3,7 @@ use std::sync::Arc;
 
 use borsh::BorshSerialize;
 
-use near_crypto::{InMemorySigner, KeyType, PublicKey, Signature, Signer};
+use near_crypto_v01::{InMemorySigner, KeyType, PublicKey, Signature, Signer};
 
 use crate::block::{Approval, ApprovalInner, BlockHeader};
 use crate::challenge::ChallengeBody;
@@ -52,17 +52,17 @@ pub trait ValidatorSigner: Sync + Send {
     fn compute_vrf_with_proof(
         &self,
         data: &[u8],
-    ) -> (near_crypto::vrf::Value, near_crypto::vrf::Proof);
+    ) -> (near_crypto_v01::vrf::Value, near_crypto_v01::vrf::Proof);
 
     /// Used by test infrastructure, only implement if make sense for testing otherwise raise `unimplemented`.
-    fn write_to_file(&self, path: &Path) -> std::io::Result<()>;
+    fn write_to_file(&self, path: &Path);
 }
 
 /// Test-only signer that "signs" everything with 0s.
 /// Don't use in any production or code that requires signature verification.
 #[derive(smart_default::SmartDefault)]
 pub struct EmptyValidatorSigner {
-    #[default("test".parse().unwrap())]
+    #[default(AccountId::test_account())]
     account_id: AccountId,
 }
 
@@ -114,11 +114,11 @@ impl ValidatorSigner for EmptyValidatorSigner {
     fn compute_vrf_with_proof(
         &self,
         _data: &[u8],
-    ) -> (near_crypto::vrf::Value, near_crypto::vrf::Proof) {
+    ) -> (near_crypto_v01::vrf::Value, near_crypto_v01::vrf::Proof) {
         unimplemented!()
     }
 
-    fn write_to_file(&self, _path: &Path) -> std::io::Result<()> {
+    fn write_to_file(&self, _path: &Path) {
         unimplemented!()
     }
 }
@@ -182,7 +182,7 @@ impl ValidatorSigner for InMemoryValidatorSigner {
     }
 
     fn sign_approval(&self, inner: &ApprovalInner, target_height: BlockHeight) -> Signature {
-        self.signer.sign(&Approval::get_data_for_sig(inner, target_height))
+        self.signer.sign(&Approval::get_data_for_sig(&inner, target_height))
     }
 
     fn sign_challenge(&self, challenge_body: &ChallengeBody) -> (CryptoHash, Signature) {
@@ -197,18 +197,18 @@ impl ValidatorSigner for InMemoryValidatorSigner {
         peer_id: &PeerId,
         epoch_id: &EpochId,
     ) -> Signature {
-        let hash = AnnounceAccount::build_header_hash(account_id, peer_id, epoch_id);
+        let hash = AnnounceAccount::build_header_hash(&account_id, &peer_id, epoch_id);
         self.signer.sign(hash.as_ref())
     }
 
     fn compute_vrf_with_proof(
         &self,
         data: &[u8],
-    ) -> (near_crypto::vrf::Value, near_crypto::vrf::Proof) {
+    ) -> (near_crypto_v01::vrf::Value, near_crypto_v01::vrf::Proof) {
         self.signer.compute_vrf_with_proof(data)
     }
 
-    fn write_to_file(&self, path: &Path) -> std::io::Result<()> {
-        self.signer.write_to_file(path)
+    fn write_to_file(&self, path: &Path) {
+        self.signer.write_to_file(path);
     }
 }
